@@ -3,7 +3,7 @@ module md_gpp_pmodel
   ! Module containing a wrapper for using the P-model photosynthesis
   ! scheme.
   !----------------------------------------------------------------
-  use md_params_core, only: nmonth, npft, nlu, c_molmass, h2o_molmass, maxgrid, ndayyear, kTkelvin, dummy
+  use md_params_core
   use md_tile_pmodel, only: tile_type, tile_fluxes_type
   use md_interface_pmodel, only: myinterface
   use md_forcing_pmodel, only: climate_type, vegcover_type
@@ -42,14 +42,9 @@ module md_gpp_pmodel
   type(paramstype_gpp) :: params_gpp
   type(pftparamstype_gpp), dimension(npft) :: params_pft_gpp
 
-  !----------------------------------------------------------------
-  ! Module-specific state variables
-  !----------------------------------------------------------------
-  real, dimension(npft) :: dassim           ! daily leaf-level assimilation rate (per unit leaf area) [gC/m2/d]
-
 contains
 
-  subroutine gpp( tile, tile_fluxes, co2, climate, vegcover, grid, init, in_ppfd)
+  subroutine gpp( tile, tile_fluxes, co2, climate, grid, init, in_ppfd)
     !//////////////////////////////////////////////////////////////////
     ! Wrapper function to call to P-model. 
     ! Calculates meteorological conditions with memory based on daily
@@ -65,7 +60,6 @@ contains
     type(tile_fluxes_type), dimension(nlu), intent(inout) :: tile_fluxes
     real, intent(in)    :: co2                               ! atmospheric CO2 (ppm)
     type(climate_type)  :: climate
-    type(vegcover_type) :: vegcover
     type(gridtype)      :: grid
     logical, intent(in) :: init                              ! is true on the very first simulation day (first subroutine call of each gridcell)
     logical, intent(in) :: in_ppfd                           ! whether to use PPFD from forcing or from SPLASH output
@@ -75,7 +69,6 @@ contains
     type(climate_type)   :: climate_acclimation     ! list of climate variables to which P-model calculates acclimated traits
     integer    :: pft
     integer    :: lu
-    real       :: iabs
     real       :: soilmstress
     real       :: kphio_temp          ! quantum yield efficiency after temperature influence
     real       :: tk
@@ -85,11 +78,6 @@ contains
     real, save :: temp_memory
     real, save :: patm_memory
     real, save :: ppfd_memory
-
-    real, save :: tmin_memory     ! for low temperature stress
-
-    ! xxx test
-    real :: a_c, a_j, a_returned, fact_jmaxlim
     integer, save :: count
 
     !----------------------------------------------------------------
@@ -132,7 +120,7 @@ contains
       ! Low-temperature effect on quantum yield efficiency 
       !----------------------------------------------------------------
       ! take the instananeously varying temperature for governing quantum yield variations
-      if (params_pft_gpp(pft)%kphio_par_a == 0.0) then
+      if (abs(params_pft_gpp(pft)%kphio_par_a) < eps) then
         kphio_temp = params_pft_gpp(pft)%kphio
       else
         kphio_temp = calc_kphio_temp( climate%dtemp, &
@@ -187,8 +175,7 @@ contains
       !----------------------------------------------------------------
       soilmstress = calc_soilmstress( tile(1)%soil%phy%wcont, &
                                       params_gpp%soilm_thetastar, &
-                                      params_gpp%soilm_betao, &
-                                      params_pft_plant(1)%grass )
+                                      params_gpp%soilm_betao )
 
       !----------------------------------------------------------------
       ! GPP
@@ -489,12 +476,6 @@ contains
     !////////////////////////////////////////////////////////////////
     ! Subroutine reads module-specific parameters from input file.
     !----------------------------------------------------------------
-    ! local variables
-    integer :: pft
-
-    !----------------------------------------------------------------
-    ! PFT-independent parameters
-    !----------------------------------------------------------------
     ! unit cost of carboxylation, b/a' in Eq. 3 (Stocker et al., 2020 GMD)
     params_gpp%beta = myinterface%params_calib%beta_unitcostratio ! 146.000000
 
